package com.benyanyi.permissionlib.shortcut;

import android.app.AppOpsManager;
import android.content.ContentResolver;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.database.Cursor;
import android.net.Uri;
import android.os.Build;
import android.text.TextUtils;

import androidx.annotation.NonNull;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * @author DELL
 * @date 2022/12/16 13:05
 * @user yangjiewei
 * @email ben@yanyi.red
 * @overview
 */
public class ShortcutPermissionChecker {

    public static boolean checkOnEMUI(@NonNull Context context) {
        Intent intent = new Intent("com.android.launcher.action.INSTALL_SHORTCUT");
        try {
            Class<?> PermissionManager = Class.forName("com.huawei.hsm.permission.PermissionManager");
            Method canSendBroadcast = PermissionManager.getDeclaredMethod("canSendBroadcast", Context.class, Intent.class);
            Object invoke = canSendBroadcast.invoke(PermissionManager, context, intent);
            if (invoke != null) {
                return (boolean) invoke;
            } else {//未知
                return true;
            }
        } catch (ClassNotFoundException e) {//Multiple-catch require API level 19 未知
            return true;
        } catch (NoSuchMethodException e) {//未知
            return true;
        } catch (IllegalAccessException e) {//未知
            return true;
        } catch (InvocationTargetException e) {//未知
            return true;
        } catch (Exception e) {//未知
            return true;
        }
    }

    public static boolean checkOnVIVO(@NonNull Context context) {
        ContentResolver contentResolver = context.getContentResolver();
        if (contentResolver == null) {
            return true;
        }
        Cursor query = null;
        try {
            Uri parse = Uri.parse("content://com.bbk.launcher2.settings/favorites");
            query = contentResolver.query(parse, null, null, null, null);
            if (query == null) {//未知
                return true;
            }

            while (query.moveToNext()) {
                String titleByQueryLauncher = query.getString(query.getColumnIndexOrThrow("title"));
                if (!TextUtils.isEmpty(titleByQueryLauncher) && titleByQueryLauncher.equals(getAppName(context))) {
                    int value = query.getInt(query.getColumnIndexOrThrow("shortcutPermission"));
                    if (value == 1 || value == 17) {
                        return false;
                    } else if (value == 16) {
                        return true;
                    } else if (value == 18) {//未知 问
                        return true;
                    }
                }
            }
        } catch (Exception e) {//未知
        } finally {
            if (query != null) {
                query.close();
            }
        }
        return true;
    }

    public static boolean checkOnMIUI(@NonNull Context context) {
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.KITKAT) {//未知
            return true;
        }
        try {
            AppOpsManager mAppOps = (AppOpsManager) context.getSystemService(Context.APP_OPS_SERVICE);
            String pkg = context.getApplicationContext().getPackageName();
            int uid = context.getApplicationInfo().uid;
            Class<?> appOpsClass = Class.forName(AppOpsManager.class.getName());
            Method checkOpNoThrowMethod = appOpsClass.getDeclaredMethod("checkOpNoThrow", Integer.TYPE, Integer.TYPE, String.class);
            Object invoke = checkOpNoThrowMethod.invoke(mAppOps, 10017, uid, pkg);//the ops of INSTALL_SHORTCUT is 10017
            if (invoke == null) {//未知
                return true;
            }
            String result = invoke.toString();
            switch (result) {
                case "0":
                    return true;
                case "1":
                    return false;
                case "5"://未知 问
                    return true;
            }
        } catch (Exception e) {//未知
            return true;
        }
        return true;
    }

    public static boolean checkOnOPPO(@NonNull Context context) {
        ContentResolver contentResolver = context.getContentResolver();
        if (contentResolver == null) {//未知
            return true;
        }
        Uri parse = Uri.parse("content://settings/secure/launcher_shortcut_permission_settings");
        Cursor query = contentResolver.query(parse, null, null, null, null);
        if (query == null) {//未知
            return true;
        }
        try {
            String pkg = context.getApplicationContext().getPackageName();
            while (query.moveToNext()) {
                String value = query.getString(query.getColumnIndex("value"));
                if (!TextUtils.isEmpty(value)) {
                    if (value.contains(pkg + ", 1")) {
                        return true;
                    }
                    if (value.contains(pkg + ", 0")) {
                        return false;
                    }
                }
            }
            return true;
        } catch (Exception e) {//未知
            return true;
        } finally {
            query.close();
        }
    }

    private static String getAppName(Context context) {
        try {
            PackageManager pm = context.getPackageManager();
            PackageInfo pi = pm.getPackageInfo(context.getApplicationContext().getPackageName(), 0);
            return pi == null ? null : pi.applicationInfo.loadLabel(pm).toString();
        } catch (PackageManager.NameNotFoundException e) {
            return "";
        }
    }
}
